 ;===============================================================================================================
 ; U-235 SoundEngine
 ; Demonstration code
 ;
 ; Written by LinkoVitch 2011
 ; Copyright 2011 U-235  http://www.u-235.co.uk
 ; 
 ; Please read LICENSE.TXT for licensing information
 ;===============================================================================================================

 ; This code is to illustrate how to utilize the U-235 SE within your own projects.  For further information read
 ; the included documentation
 
; ====================================================================================================== INCLUDES
 
.INCLUDE "jaguar.inc"

;=========================================================================================================== MAIN
 
.text
.68000

; Setup the Jag

	move.l	#INITSTACK,a7	; Setup a stack (point to the end of RAM)
	
	jsr 	jaginit				; Initialise the Jaguar (roar 64bit powah!)
	
	jsr		joypadinit			; Setup the Jagpad for reading
	lea		padlist1,a0			; these lines set the routines to be called when that button press is detected
	move.l	#pad1,PAD_1(a0)
	move.l	#pad2,PAD_2(a0)
	move.l	#pad3,PAD_3(a0)
	move.l	#pad4,PAD_4(a0)
	move.l	#pad5,PAD_5(a0)
	move.l	#pad6,PAD_6(a0)
	move.l	#pad7,PAD_7(a0)

.EXTERN bufferptr

	jsr 	InitVBInt			; Initialise the VBI - we will use this for polling the pad
	
.EXTERN dspcode
	
	;-------------------------- Copy the SoundEngine to the DSP RAM
	
	move.w	#2048,d0
	lea		dspcode,a0
	move.l	#D_RAM,a1

.loop:
	move.l	(a0)+,(a1)+
	dbra.w	d0,.loop
		
	;-------------------------------------- Setup the playback rate
	
	.EXTERN	U235SE_period_ptr
	lea		U235SE_period_ptr,a0
	move.l	#period_table,(a0)
		
	move.w	#C_VIDENA,INT1		; Enable the VBI 
	move.w	#$100,JOYSTICK		; enable sound

	; enable DSP
		
	move.l	#D_RAM,D_PC			; Set up D_PC (DSP's program counter)
	move.l	#RISCGO,D_CTRL		; Go, baby go!

	jsr		modinit					; Initialize the module - populates the sample table in the mod player with sample 
									;						  info so that the player can do it's thang
	
	;=================================================================
	;-------------------------------------- Setup the SFX Sample bank
	;=================================================================
	
	.EXTERN U235SE_ptr_sample_bank
	lea		U235SE_ptr_sample_bank,a0	; address of the sample bank pointer in the SoundEngine
	move.l	#sample_bank,(a0)			; store the address of where the bank is in the pointer
	
	.EXTERN U235SE_playmod			
	move.l	#1,U235SE_playmod				; Enable mod playback

;================================================================================================================ MAIN LOOP
mainloop:

	jsr			readpad1		; poll joypad
	
.wait:
	cmpi.w	#1,VBIFLAG			; Has a VBI occured?
	bne		.wait				; nope
	clr.w	VBIFLAG				; yep, clear the flag and run the main loop again
	
	bra.s	mainloop

.EXTERN U235SE_sfxplaylist_ptr

;======================================================================================================== JAGPAD FUNCTIONS
	
;------------------------------------------------------------------------------------------------------------------- PAD 1
pad1:
	move.l	#0,U235SE_playmod		; setting playmod to 0 will stop module playback
	rts

;------------------------------------------------------------------------------------------------------------------- PAD 2	
		
pad2:
	move.l	#1,U235SE_playmod		; Setting playmod to 1 will enable mono module playback (2 enables stereo)
	rts

;------------------------------------------------------------------------------------------------------------------- PAD 3	
	
pad3:
	movem.l	d0/a0-a1,-(sp)
	lea		playlist,a0			; The address of the sound effect playlist
	lea		U235SE_sfxplaylist_ptr,a1

	; This is command #6 - play note
	; Commands are 32 bit words made up typically of YYYYXXVC
	; YYYY = Data for command
	; XX   = Data for command
	; V    = Voice to act on
	; C    = Command number
	
	move.l	#$00202046,(a0)+	; play sample 32 on voice #4 at note #32
	
	; The above breaks down as follows:
	; $00202046
	;  YYYYXXVC
	;
	; YYYY = $0020 = 32 .. in this case the note number
	; XX   = $20   = 32 .. Sample number 32, so the 1st sample from the non module sample bank
	; V    = $4    = 4  .. Voice number 4, the 1st channel after the ones used by the mod player
	; C    = $6    = 6  .. Command 6, play note as specified.
	
	move.l	#0,(a0)				; terminate the end of the list - Or the SE will skip off through RAM making a mess
	
	move.l	#playlist,d0		; once the list is complete, we need to set the playlist pointer to point at it
	move.l	d0,(a1)				; As soon as this is written the SE will start working through the list of commands
	movem.l	(sp)+,d0/a0-a1
	rts

;------------------------------------------------------------------------------------------------------------------- PAD 4	
	
pad4:
	movem.l	d0/a0-a1,-(sp)
	lea		playlist,a0
	lea		U235SE_sfxplaylist_ptr,a1

	move.l	#$00252056,(a0)+	; same as PAD3 but voice #5 and note 37

	move.l	#0,(a0)
	
	move.l	#playlist,d0
	move.l	d0,(a1)
	movem.l	(sp)+,d0/a0-a1
	rts
	
;------------------------------------------------------------------------------------------------------------------- PAD 5		
	
pad5:
	movem.l	d0/a0-a1,-(sp)
	lea		playlist,a0
	lea		U235SE_sfxplaylist_ptr,a1

	move.l	#$002a2066,(a0)+	; Same as PAD 3 but voice #6 

	move.l	#0,(a0)
	
	move.l	#playlist,d0
	move.l	d0,(a1)
	movem.l	(sp)+,d0/a0-a1
	rts


;------------------------------------------------------------------------------------------------------------------- PAD 6		
	
pad6:
	movem.l	d0/a0-a1,-(sp)
	lea		playlist,a0
	lea		U235SE_sfxplaylist_ptr,a1
	
	move.l	#$002f2076,(a0)+
	
	move.l	#0,(a0)
	
	move.l	#playlist,d0
	move.l	d0,(a1)
	movem.l	(sp)+,d0/a0-a1
	rts

;------------------------------------------------------------------------------------------------------------------- PAD 7		
pad7:
	movem.l	d0/a0-a1,-(sp)
	lea		playlist,a0
	lea		U235SE_sfxplaylist_ptr,a1

	; play the sample on all 8 voices at the same time
	
	move.l	#$192006,(a0)+
	move.l	#$192016,(a0)+
	move.l	#$192026,(a0)+
	move.l	#$192036,(a0)+
	move.l	#$192046,(a0)+
	move.l	#$192056,(a0)+
	move.l	#$192066,(a0)+
	move.l	#$192076,(a0)+
	
	move.l	#0,(a0)
	
	move.l	#playlist,d0
	move.l	d0,(a1)
	movem.l	(sp)+,d0/a0-a1
	rts

;==================================================================================================================== INCLUDES	

.INCLUDE "setup.s"		; Jaguar setup routines
.INCLUDE "vbi.s"		; Vertical Blank setup / routines
.INCLUDE "joypad.s"		; For reading the joypad

.INCLUDE "sequence.s"	; Bits and pieces left over in 68K land for setup of mod playback

;======================================================================================================================== DATA
.LONG	
playlist:	dc.l	0,0,0,0,0,0,0,0
	
oldpos:		dc.l	$ff

sample_bank:
	; Sample #1
		dc.l	sample1		; base address of sample in RAM
		dc.l	sample1_end	; End of sample
		dc.l	0			; RBASE
		dc.l	0			; REND
		dc.l	64			; <null word> | Fine tune | Volume
		dc.l	2990		; Default play rate (only used in SFX)
							; 24 bytes per sample.  744 bytes total

;============================================================================================================ PERIOD RATIOS

.EVEN
period_table:
.INCLUDE	"period24.s"

;=========================================================================================================== Example Sample

sample1:
.INCBIN "orch.sam"
	
sample1_end:	dc.b	0
.EVEN

;=========================================================================================================== Example Module

module:
.INCBIN "alf.mod"


		
		